/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.api.pattern;

import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag4j.types.scheduling.TimeConstraintType;
import org.ogf.schemas.graap.wsAgreement.TermCompositorType;

/**
 * AdvanceReservationPattern
 * 
 * An advance reservation pattern includes a service description term that defines the earliest start time of
 * the overall service. The example below illustrates the structure of an advance reservation template.
 * 
 * <pre>
 * {@code
 * <wsag:Template wsag:TemplateId="1"
 *  xmlns:wsag="http://schemas.ggf.org/graap/2007/03/ws-agreement">
 *  <wsag:Name>ADAVANCE_RESERVATION_TEMPLATE</wsag:Name>
 *  <wsag:Context>
 *      <wsag:ServiceProvider>AgreementResponder</wsag:ServiceProvider>
 *      <wsag:TemplateId>1</wsag:TemplateId>
 *      <wsag:TemplateName>ADAVANCE_RESERVATION_TEMPLATE</wsag:TemplateName>
 *  </wsag:Context>
 *  <wsag:Terms>
 *      <wsag:All>
 *          <wsag:ServiceDescriptionTerm wsag:Name="SDT_1"
 *              wsag:ServiceName="SERVICE_1">
 *              (Put your service description terms here) ....
 *          </wsag:ServiceDescriptionTerm>
 *          ...
 *          <wsag:ServiceDescriptionTerm wsag:Name="TIME_CONSTRAINT_SDT"
 *              wsag:ServiceName="SERVICE_1">
 *              <wsag-tc:TimeConstraint
 *                  xmlns:wsag-tc="http://schemas.wsag4j.org/2009/07/wsag4j-scheduling-extensions">
 *                  <wsag-tc:StartTime>2010-06-24T02:00:00</wsag-tc:StartTime>
 *              </wsag-tc:TimeConstraint>
 *          </wsag:ServiceDescriptionTerm>
 *      </wsag:All>
 *  </wsag:Terms>
 *  <wsag:CreationConstraints />
 * </wsag:Template>
 * }
 * </pre>
 * 
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AdvanceReservationPattern extends AbstractPattern
{

    private TimeConstraintType timeConstraint;

    private String timeConstraintXPath =
        "declare namespace wsag-tc='http://schemas.wsag4j.org/2009/07/wsag4j-scheduling-extensions';"
            + "declare namespace wsag='http://schemas.ggf.org/graap/2007/03/ws-agreement';"
            + "$this/wsag:ServiceDescriptionTerm[@wsag:Name = 'TIME_CONSTRAINT_SDT']/wsag-tc:TimeConstraint";

    /**
     * Instantiates the advance reservation pattern on a given term compositor.
     * 
     * @param termCompositor
     *            the term compositor
     */
    public AdvanceReservationPattern( TermCompositorType termCompositor )
    {
        super( termCompositor );

        initialize();
    }

    /**
     * initializes the pattern object after creation
     */
    protected void initialize()
    {
        XmlObject[] arConstraints = getTermCompositor().selectPath( getTimeConstraintXPath() );

        if ( arConstraints.length != 1 )
        {
            throw new IllegalStateException(
                "Invalid number of advance reservation time constraints in template." );
        }

        timeConstraint = (TimeConstraintType) arConstraints[0];
    }

    /**
     * @return the timeConstraint
     */
    public TimeConstraintType getTimeConstraint()
    {
        return timeConstraint;
    }

    /**
     * @param timeConstraintXPath
     *            the timeConstraintXPath to set
     */
    public void setTimeConstraintXPath( String timeConstraintXPath )
    {
        this.timeConstraintXPath = timeConstraintXPath;
    }

    /**
     * @return the timeConstraintXPath
     */
    public String getTimeConstraintXPath()
    {
        return timeConstraintXPath;
    }

}
