/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.api.pattern;

import java.text.MessageFormat;

import javax.xml.namespace.QName;

import org.apache.xmlbeans.XmlObject;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ApplicationType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionType;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.schemas.graap.wsAgreement.ServiceDescriptionTermType;
import org.ogf.schemas.graap.wsAgreement.TermCompositorType;

/**
 * ChoosableComputeApplicationPattern
 * 
 * The term compositor contains a set of compute applications, where the client must choose on application out
 * of the set.
 * <p>
 * 
 * The protocol of this pattern is as follows. First one must select an application using
 * {@link #selectApplication(ApplicationType)} or {@link #selectApplication(String, String)}. This service
 * selection is instantly registered in the underlying term compositor.
 * <p>
 * 
 * Subsequent modifications to the selected applications are possible by using functions such as
 * {@link #getSelectedApplication()} or {@link #getSelectedJobDefinition()}. These modifications are not
 * registered in the term compositor until {@link #updateSelectedApplication()} is executed. <br>
 * 
 * <pre>
 * {@code
 * <wsag:Template wsag:TemplateId="1"
 *     xmlns:wsag="http://schemas.ggf.org/graap/2007/03/ws-agreement">
 *     <wsag:Name>ComputeApplicationTemplate</wsag:Name>
 *     <wsag:Context>
 *         <wsag:ServiceProvider>AgreementResponder</wsag:ServiceProvider>
 *         <wsag:TemplateId>1</wsag:TemplateId>
 *         <wsag:TemplateName>ComputeApplicationTemplate</wsag:TemplateName>
 *     </wsag:Context>
 *     <wsag:Terms>
 *         <wsag:All>
 *             <wsag:ServiceDescriptionTerm wsag:Name="SDT_1"
 *                 wsag:ServiceName="SERVICE_1">
 *                 (Put your service description terms here ...)
 *             </wsag:ServiceDescriptionTerm>
 *             ...
 *             <wsag:ExactlyOne>
 *                 <wsag:ServiceDescriptionTerm wsag:Name="APPLICATION_SDT"
 *                    wsag:ServiceName="CHANGE_ME">
 *                     <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                         <jsdl:JobDescription>
 *                             <jsdl:Application>
 *                                 <jsdl:ApplicationName>MyApp</jsdl:ApplicationName>
 *                                 <jsdl:ApplicationVersion></jsdl:ApplicationVersion>
 *                                 <jsdl:Description>Description</jsdl:Description>
 *                             </jsdl:Application>
 *                         </jsdl:JobDescription>
 *                     </jsdl:JobDefinition>
 *                 </wsag:ServiceDescriptionTerm>
 *                 <wsag:ServiceDescriptionTerm wsag:Name="APPLICATION_SDT"
 *                     wsag:ServiceName="CHANGE_ME">
 *                     <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                         <jsdl:JobDescription>
 *                             <jsdl:Application>
 *                                 <jsdl:ApplicationName></jsdl:ApplicationName>
 *                                 <jsdl:ApplicationVersion></jsdl:ApplicationVersion>
 *                                 <jsdl:Description>Description</jsdl:Description>
 *                             </jsdl:Application>
 *                         </jsdl:JobDescription>
 *                     </jsdl:JobDefinition>
 *                 </wsag:ServiceDescriptionTerm>
 *             </wsag:ExactlyOne>
 *         </wsag:All>
 *     </wsag:Terms>
 *     <wsag:CreationConstraints />
 * </wsag:Template>
 * }
 * </pre>
 * 
 * @author Oliver Waeldrich
 * @author Dominic Battre
 * 
 * 
 */
public class ChoosableComputeApplicationPattern extends AbstractPattern
{

    private static final QName EXACTLY_ONE_QNAME = new QName( WsagConstants.NAMESPACE_URI, "ExactlyOne" );

    private ServiceDescriptionTermType selectedApplicationSDT;

    private ServiceDescriptionTermType[] applicationSDTs;

    private ApplicationType[] applicationDefinitions;

    private String appliationDescXPath =
        "declare namespace wsag='http://schemas.ggf.org/graap/2007/03/ws-agreement';"
            + "$this/wsag:ServiceDescriptionTerm[@wsag:Name = 'APPLICATION_SDT' ]";

    private String appliationXPath = "declare namespace jsdl='http://schemas.ggf.org/jsdl/2005/11/jsdl';"
        + "$this/jsdl:JobDefinition/jsdl:JobDescription/jsdl:Application";

    /**
     * Instantiates the pattern for the given term compositor.
     * 
     * @param termCompositor
     *            the term compositor
     */
    public ChoosableComputeApplicationPattern( TermCompositorType termCompositor )
    {
        super( termCompositor );

        //
        // check that the pattern is applied to an ExactlyOne term compositor
        //
        String localName = termCompositor.getDomNode().getLocalName();
        String namespaceURI = termCompositor.getDomNode().getNamespaceURI();

        QName compositorName = new QName( namespaceURI, localName );

        if ( !EXACTLY_ONE_QNAME.equals( compositorName ) )
        {
            String msgError =
                "Tried to apply {0} to {1} term compositor. "
                    + "This pattern can only be applied to wsag:ExactlyOne term compositor.";
            String error =
                MessageFormat.format( msgError, new Object[] { getClass().getName(), compositorName } );
            throw new IllegalStateException( error );
        }

        if ( ( termCompositor.getAllArray().length > 0 ) || ( termCompositor.getExactlyOneArray().length > 0 )
            || ( termCompositor.getOneOrMoreArray().length > 0 ) )
        {

            String msgError =
                "A ChoosableComputeApplicationPattern must not contain any other term compositor.";
            throw new IllegalStateException( msgError );
        }

        if ( ( termCompositor.getGuaranteeTermArray().length > 0 )
            || ( termCompositor.getServicePropertiesArray().length > 0 )
            || ( termCompositor.getServiceReferenceArray().length > 0 ) )
        {

            String msgError =
                "A ChoosableComputeApplicationPattern must not contain any Service Properties, "
                    + "Service References, or Guarantee terms.";
            throw new IllegalStateException( msgError );
        }

        initialize();
    }

    private void initialize()
    {
        //
        // select the application SDTs and the applications
        //
        XmlObject[] applicationDescriptions = getTermCompositor().selectPath( getAppliationDescXPath() );

        applicationSDTs = new ServiceDescriptionTermType[applicationDescriptions.length];
        applicationDefinitions = new ApplicationType[applicationDescriptions.length];

        for ( int i = 0; i < applicationDescriptions.length; i++ )
        {
            applicationSDTs[i] = (ServiceDescriptionTermType) applicationDescriptions[i].copy();

            XmlObject[] selectedApps = applicationDescriptions[i].selectPath( getAppliationXPath() );
            if ( selectedApps.length != 1 )
            {
                String msgError = "APPLICATION_SDT must contain exactly 1 JSDL application definition.";
                throw new IllegalStateException( msgError );
            }

            applicationDefinitions[i] = (ApplicationType) selectedApps[0].copy();
        }

        //
        // select application and set the chosen application for this term compositor
        //
        setSelectedApplication( applicationSDTs[0] );
    }

    /**
     * 
     * @return the job definition of the selected application SDT
     */
    public JobDefinitionType getSelectedJobDefinition()
    {
        final QName jobDefQName = JobDefinitionDocument.type.getDocumentElementName();
        return (JobDefinitionType) selectedApplicationSDT.selectChildren( jobDefQName )[0];
    }

    /**
     * @return the selected application
     */
    public ApplicationType getSelectedApplication()
    {
        return (ApplicationType) selectedApplicationSDT.selectPath( getAppliationXPath() )[0];
    }

    /**
     * Selects the application in the term compositor.
     * 
     * @param application
     *            the application to select
     * @return true, if the application was selected, otherwise false.
     */
    public boolean selectApplication( ApplicationType application )
    {
        return selectApplication( application.getApplicationName(), application.getApplicationVersion() );
    }

    /**
     * Selects the application wit the given name and version in the term compositor.
     * 
     * @param name
     *            the application name
     * @param version
     *            the application version
     * @return true, if the application was selected, otherwise false.
     */
    public boolean selectApplication( String name, String version )
    {
        name = ( name == null ) ? "" : name;

        for ( int i = 0; i < applicationDefinitions.length; i++ )
        {
            if ( name.equals( applicationDefinitions[i].getApplicationName() ) )
            {

                boolean match = false;

                if ( version != null )
                {
                    match = version.equals( applicationDefinitions[i].getApplicationVersion() );
                }
                else
                {
                    match = applicationDefinitions[i].getApplicationVersion() == null;
                }

                if ( match )
                {
                    setSelectedApplication( applicationSDTs[i] );
                    return true;
                }
            }
        }

        return false;
    }

    private void setSelectedApplication( ServiceDescriptionTermType selected )
    {
        selectedApplicationSDT = (ServiceDescriptionTermType) selected.copy();
        updateSelectedApplication();
    }

    /**
     * Updates the selected application in the term compositor. This method actually updates the XML document
     * referenced by the term compositor.
     */
    public void updateSelectedApplication()
    {
        ServiceDescriptionTermType[] appSDT = new ServiceDescriptionTermType[] { selectedApplicationSDT };
        getTermCompositor().setServiceDescriptionTermArray( appSDT );
    }

    /**
     * Lists all applications in the term compositor.
     * 
     * @return the applications contained in the term compositor.
     */
    public ApplicationType[] listApplications()
    {
        return applicationDefinitions;
    }

    /**
     * @param appliationDescXPath
     *            the appliationDescXPath to set
     */
    public void setAppliationDescXPath( String appliationDescXPath )
    {
        this.appliationDescXPath = appliationDescXPath;
    }

    /**
     * @return the appliationDescXPath
     */
    public String getAppliationDescXPath()
    {
        return appliationDescXPath;
    }

    /**
     * @param appliationXPath
     *            the appliationXPath to set
     */
    public void setAppliationXPath( String appliationXPath )
    {
        this.appliationXPath = appliationXPath;
    }

    /**
     * @return the appliationXPath
     */
    public String getAppliationXPath()
    {
        return appliationXPath;
    }

}
