/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.api.pattern;

import org.apache.xmlbeans.XmlObject;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ApplicationType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionType;
import org.ogf.schemas.graap.wsAgreement.TermCompositorType;

/**
 * ComputeApplicationPattern
 * 
 * The term compositor contains an Application SDT.
 * 
 * @author Oliver Waeldrich
 * 
 *         <pre>
 * {@code
 * <wsag:Template wsag:TemplateId="1"
 *  xmlns:wsag="http://schemas.ggf.org/graap/2007/03/ws-agreement">
 *  <wsag:Name>ADAVANCE_RESERVATION_TEMPLATE</wsag:Name>
 *  <wsag:Context>
 *      <wsag:ServiceProvider>AgreementResponder</wsag:ServiceProvider>
 *      <wsag:TemplateId>1</wsag:TemplateId>
 *      <wsag:TemplateName>ADAVANCE_RESERVATION_TEMPLATE</wsag:TemplateName>
 *  </wsag:Context>
 *  <wsag:Terms>
 *      <wsag:All>
 *          <wsag:ServiceDescriptionTerm wsag:Name="SDT_1"
 *              wsag:ServiceName="SERVICE_1">
 *              (Put your service description terms here) ....
 *          </wsag:ServiceDescriptionTerm>
 *          ...
 *          <wsag:ServiceDescriptionTerm wsag:Name="APPLICATION_SDT"
 *              wsag:ServiceName="CHANGE_ME">
 *              <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                  <jsdl:JobDescription>
 *                      <jsdl:Application>
 *                          <jsdl:ApplicationName>ApplicationName</jsdl:ApplicationName>
 *                          <jsdl:ApplicationVersion>ApplicationVersion</jsdl:ApplicationVersion>
 *                          <jsdl:Description>Description</jsdl:Description>
 *                      </jsdl:Application>
 *                  </jsdl:JobDescription>
 *              </jsdl:JobDefinition>
 *          </wsag:ServiceDescriptionTerm>
 *      </wsag:All>
 *  </wsag:Terms>
 *  <wsag:CreationConstraints />
 * </wsag:Template>
 * }
 * </pre>
 * 
 * 
 */
public class ComputeApplicationPattern extends AbstractPattern
{

    private JobDefinitionType jobDefinition;

    private static final String APPLICATION_XPATH =
        "declare namespace jsdl='http://schemas.ggf.org/jsdl/2005/11/jsdl';"
            + "declare namespace wsag='http://schemas.ggf.org/graap/2007/03/ws-agreement';"
            + "$this/wsag:ServiceDescriptionTerm[@wsag:Name = 'APPLICATION_SDT']/jsdl:JobDefinition";

    /**
     * Instantiates the pattern for the given term compositor.
     * 
     * @param termCompositor
     *            the term compositor
     */
    public ComputeApplicationPattern( TermCompositorType termCompositor )
    {
        super( termCompositor );

        initialize();
    }

    /**
     * initializes the quote object
     */
    private void initialize()
    {
        XmlObject[] jobConstraints = getTermCompositor().selectPath( getApplicationxpath() );

        if ( jobConstraints.length != 1 )
        {
            throw new IllegalStateException( "Invalid number of job descriptions in template." );
        }

        jobDefinition = (JobDefinitionType) jobConstraints[0];
    }

    /**
     * @return the selected application
     */
    public ApplicationType getSelectedApplication()
    {
        return jobDefinition.getJobDescription().getApplication();
    }

    /**
     * @return the selected job definition that contains the application
     */
    public JobDefinitionType getSelectedJobDefinition()
    {
        return jobDefinition;
    }

    /**
     * Subclasses may overwrite this method in order to provide a custom expression for selecting the
     * applications.
     * 
     * @return the XPath expression that is used to select the applications from the term compositor.
     */
    public static String getApplicationxpath()
    {
        return APPLICATION_XPATH;
    }

}
