/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.api.pattern;

import org.apache.xmlbeans.XmlObject;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ResourcesType;
import org.ogf.schemas.graap.wsAgreement.TermCompositorType;

/**
 * ComputeResourcePattern
 * 
 * The term compositor contains a RESOURCE_SDT with a JSDL document describing a resource.
 * 
 * <pre>
 * {@code
 * <wsag:Template wsag:TemplateId="1"
 *  xmlns:wsag="http://schemas.ggf.org/graap/2007/03/ws-agreement">
 *  <wsag:Name>ComputeApplicationTemplate</wsag:Name>
 *  <wsag:Context>
 *      <wsag:ServiceProvider>AgreementResponder</wsag:ServiceProvider>
 *      <wsag:TemplateId>1</wsag:TemplateId>
 *      <wsag:TemplateName>ComputeApplicationTemplate</wsag:TemplateName>
 *  </wsag:Context>
 *  <wsag:Terms>
 *      <wsag:All>
 *          <wsag:ServiceDescriptionTerm wsag:Name="SDT_1"
 *              wsag:ServiceName="SERVICE_1"> 
 *                  Put your service description terms here ...
 *          </wsag:ServiceDescriptionTerm>
 *          ...
 *          <wsag:ServiceDescriptionTerm wsag:Name="RESOURCE_SDT"
 *              wsag:ServiceName="COMPUTE_SERVICE">
 *              <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                  <jsdl:JobDescription>
 *                      <jsdl:Resources>
 *                          <jsdl:IndividualCPUTime>
 *                              <jsdl:Exact>3600.0</jsdl:Exact>
 *                          </jsdl:IndividualCPUTime>
 *                          <jsdl:IndividualCPUCount>
 *                              <jsdl:Exact>2.0</jsdl:Exact>
 *                          </jsdl:IndividualCPUCount>
 *                          <jsdl:TotalResourceCount>
 *                              <jsdl:Exact>1.0</jsdl:Exact>
 *                          </jsdl:TotalResourceCount>
 *                      </jsdl:Resources>
 *                  </jsdl:JobDescription>
 *              </jsdl:JobDefinition>
 *          </wsag:ServiceDescriptionTerm>
 * 
 *      </wsag:All>
 *  </wsag:Terms>
 *  <wsag:CreationConstraints />
 * </wsag:Template>
 * }
 * </pre>
 * 
 * @author Oliver Waeldrich
 * 
 */
public class ComputeResourcePattern extends AbstractPattern
{

    private JobDefinitionType resourceSDT;

    private final String resourceXPath = "declare namespace jsdl='http://schemas.ggf.org/jsdl/2005/11/jsdl';"
        + "declare namespace wsag='http://schemas.ggf.org/graap/2007/03/ws-agreement';"
        + "$this/wsag:ServiceDescriptionTerm[@wsag:Name = 'RESOURCE_SDT']/jsdl:JobDefinition";

    /**
     * Initiates the pattern on a given term compositor.
     * 
     * @param termCompositor
     *            the term compositor
     */
    public ComputeResourcePattern( TermCompositorType termCompositor )
    {
        super( termCompositor );
        initialize();
    }

    /**
     * initializes the pattern object after creation
     */
    protected void initialize()
    {
        //
        // find the exactly one element, that contains the application SDTs
        //
        XmlObject[] resourceSelection = getTermCompositor().selectPath( getResourceXPath() );

        if ( resourceSelection.length != 1 )
        {
            throw new IllegalStateException( "Invalid number of resource SDTs in quote." );
        }

        resourceSDT = (JobDefinitionType) resourceSelection[0];

    }

    /**
     * 
     * @return the JSDL resource section in the SDT
     */
    public ResourcesType getResourceDefinition()
    {
        return resourceSDT.getJobDescription().getResources();
    }

    /**
     * Sub classes overwrite this method in order to provide a custom XPath expression for resource SDT
     * selection.
     * 
     * @return the resourceXPath
     */
    public String getResourceXPath()
    {
        return resourceXPath;
    }

}
