/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.api.sla;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.ApplicationType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ResourcesType;
import org.ogf.graap.wsag.api.pattern.ChoosableComputeApplicationPattern;
import org.ogf.graap.wsag.api.pattern.ComputeResourcePattern;
import org.ogf.graap.wsag.api.types.TemplateType;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.TermCompositorType;

/**
 * AbstractComputeJobTemplate
 * 
 * The protocol of this class is as follows. First one must select an application using
 * {@link #selectApplication(ApplicationType)} or {@link #selectApplication(String, String)}. This service
 * selection is instantly registered in the underlying template.
 * <p>
 * 
 * Subsequent modifications to the selected applications are possible by using functions such as
 * {@link #getSelectedApplication()} or {@link #getSelectedJobDefinition()}. Modifications to the returned
 * objects are not registered in the template until {@link #updateSelectedApplication()} is executed.
 * 
 * @author Oliver Waeldrich
 * 
 */
public abstract class AbstractComputeJobTemplate extends TemplateType
{

    private ChoosableComputeApplicationPattern computeTemplate;

    private ComputeResourcePattern resourceTemplate;

    /**
     * @param template
     *            the XML template object
     */
    public AbstractComputeJobTemplate( AgreementTemplateType template )
    {
        super( template );

        //
        // use the internal XML object for creating the pattern
        //
        template = getXMLObject();

        TermCompositorType compositor = template.getTerms().getAll().getExactlyOneArray( 0 );
        computeTemplate = new ChoosableComputeApplicationPattern( compositor );
        resourceTemplate = new ComputeResourcePattern( template.getTerms().getAll() );
    }

    /**
     * @return the resource definition
     * @see ComputeResourcePattern#getResourceDefinition()
     */
    public ResourcesType getResourceDefinition()
    {
        return resourceTemplate.getResourceDefinition();
    }

    /**
     * @return the selected application
     * @see ChoosableComputeApplicationPattern#getSelectedApplication()
     */
    public ApplicationType getSelectedApplication()
    {
        return computeTemplate.getSelectedApplication();
    }

    /**
     * @return a list of all available application
     * @see ChoosableComputeApplicationPattern#listApplications()
     */
    public ApplicationType[] listApplications()
    {
        return computeTemplate.listApplications();
    }

    /**
     * @param application
     *            the application to select
     * @return true if the application was selected, otherwise false
     * @see ChoosableComputeApplicationPattern#selectApplication(org.ggf.schemas.jsdl.x2005.x11.jsdl.ApplicationType)
     */
    public boolean selectApplication( ApplicationType application )
    {
        return computeTemplate.selectApplication( application );
    }

    /**
     * @param name
     *            application name
     * @param version
     *            application version
     * @return true if the chosen application was selected, otherwise false
     * @see ChoosableComputeApplicationPattern#selectApplication(java.lang.String, java.lang.String)
     */
    public boolean selectApplication( String name, String version )
    {
        return computeTemplate.selectApplication( name, version );
    }

    /**
     * 
     * @see ChoosableComputeApplicationPattern#updateSelectedApplication()
     */
    public void updateSelectedApplication()
    {
        computeTemplate.updateSelectedApplication();
    }

    /**
     * @return the job definition that contains the currently selected application
     * @see ChoosableComputeApplicationPattern#getSelectedJobDefinition()
     */
    public JobDefinitionType getSelectedJobDefinition()
    {
        return computeTemplate.getSelectedJobDefinition();
    }
}