/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.interpreter;

import org.ogf.graap.wsag.interpreter.expr.AllExpression;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.AgreementType;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermType;
import org.ogf.schemas.graap.wsAgreement.ServiceDescriptionTermType;
import org.ogf.schemas.graap.wsAgreement.ServicePropertiesType;
import org.ogf.schemas.graap.wsAgreement.ServiceReferenceType;
import org.ogf.schemas.graap.wsAgreement.TermTreeType;
import org.ogf.schemas.graap.wsAgreement.TermType;

/**
 * The agreement interpreter evaluates the agreement language in the agreement terms property and checks
 * whether or not the agreement is valid with respect to term composition.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AgreementInterpreter
{
    /**
     * Evaluates the term composition in the given template and returns the template if the terms in the
     * template are valid with respect to term composition. Otherwise an exception is thrown.
     * 
     * @param template
     *            the template to evaluate
     * @return the valid template
     * @throws InterpreterException
     *             error during term processing
     */
    public AgreementTemplateType process( AgreementTemplateType template ) throws InterpreterException
    {
        return (AgreementTemplateType) interpretAgreementType( template );
    }

    /**
     * Evaluates the term composition in the given agreement and returns the agreement object if the terms are
     * valid with respect to term composition.
     * 
     * @param agreement
     *            the agreement to evaluate
     * @return the valid agreement
     * @throws InterpreterException
     *             error during term processing
     */
    public AgreementType process( AgreementType agreement ) throws InterpreterException
    {
        return interpretAgreementType( agreement );
    }

    private AgreementType interpretAgreementType( AgreementType toProcess ) throws InterpreterException
    {

        toProcess = (AgreementType) toProcess.copy();

        AllExpression all = new AllExpression( toProcess.getTerms().getAll() );

        //
        // interpret the all expression
        //
        TermType[] terms = all.interpret( new InterpreterContext() );

        TermTreeType processed = TermTreeType.Factory.newInstance();
        processed.addNewAll();

        for ( int i = 0; i < terms.length; i++ )
        {
            if ( terms[i] instanceof ServiceDescriptionTermType )
            {
                processed.getAll().addNewServiceDescriptionTerm().set( terms[i] );
            }
            if ( terms[i] instanceof GuaranteeTermType )
            {
                processed.getAll().addNewGuaranteeTerm().set( terms[i] );
            }
            if ( terms[i] instanceof ServicePropertiesType )
            {
                processed.getAll().addNewServiceProperties().set( terms[i] );
            }
            if ( terms[i] instanceof ServiceReferenceType )
            {
                processed.getAll().addNewServiceReference().set( terms[i] );
            }
        }

        //
        // reinitialize the term tree
        //
        toProcess.setTerms( processed );

        return toProcess;
    }
}
