/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.interpreter.expr;

import java.util.Vector;

import org.ogf.graap.wsag.interpreter.IExpression;
import org.ogf.graap.wsag.interpreter.InterpreterContext;
import org.ogf.graap.wsag.interpreter.InterpreterException;
import org.ogf.schemas.graap.wsAgreement.TermCompositorType;
import org.ogf.schemas.graap.wsAgreement.TermType;

/**
 * WS-Agreement allows to logically compose terms in an agreement using a simple expression language. Terms
 * can be composed using logical AND (All), OR (OneOrMore), and XOR (ExacltyOne) operations in the agreement
 * terms. This abstract expression implementation is the base class for the concrete implementations. It is
 * used by the {@link org.ogf.graap.wsag.interpreter.AgreementInterpreter} that evaluates the validity of the
 * agreement terms in an offer with respect to the expression language.
 * 
 * @see org.ogf.graap.wsag.interpreter.AgreementInterpreter
 * 
 * @author Oliver Waeldrich
 * 
 */
public abstract class AbstractExpression implements IExpression
{

    private TermCompositorType expression;

    private Vector<TermType> result;

    /**
     * Creates an expression for a given term compositor.
     * 
     * @param term
     *            the agreement term compositor
     */
    public AbstractExpression( TermCompositorType term )
    {
        this.expression = term;
    }

    /**
     * template method that implements a specific interpretation strategy. The strategy interprets a set of
     * terms and returns a result, i.e. the selected term in an ExactlyOne statement.
     * 
     * @param terms
     *            the terms to interpret
     * @param context
     *            the interpreter context
     * @return the strategy result, i.e. a filtered term list
     * 
     * @throws InterpreterException
     *             indicates an error during the term interpretation
     */
    protected abstract TermType[] interpret( TermType[] terms, InterpreterContext context )
        throws InterpreterException;

    /**
     * {@inheritDoc}
     */
    public TermType[] interpret( InterpreterContext context ) throws InterpreterException
    {
        result = new Vector<TermType>();

        //
        // interpret the expressions
        //
        interpretAllExpression( expression.getAllArray(), context );
        interpretExactlyOneExpression( expression.getExactlyOneArray(), context );
        interpretOneOrMoreExpression( expression.getOneOrMoreArray(), context );

        //
        // interpret the terms
        //
        interpretTerms( expression.getServiceDescriptionTermArray(), context );
        interpretTerms( expression.getGuaranteeTermArray(), context );
        interpretTerms( expression.getServicePropertiesArray(), context );
        interpretTerms( expression.getServiceReferenceArray(), context );

        //
        // now let the current expression handle the interpretation result
        //
        TermType[] interpretedTerms = (TermType[]) result.toArray( new TermType[result.size()] );

        return interpret( interpretedTerms, context );
    }

    private void interpretAllExpression( TermCompositorType[] compositor, InterpreterContext context )
        throws InterpreterException
    {

        for ( int i = 0; i < compositor.length; i++ )
        {
            AllExpression allExpression = new AllExpression( compositor[i] );
            TermType[] interpretionResult = allExpression.interpret( context );
            for ( int j = 0; j < interpretionResult.length; j++ )
            {
                result.add( interpretionResult[j] );
            }
        }

    }

    private void interpretExactlyOneExpression( TermCompositorType[] compositor, InterpreterContext context )
        throws InterpreterException
    {

        for ( int i = 0; i < compositor.length; i++ )
        {
            ExactlyOneExpression allExpression = new ExactlyOneExpression( compositor[i] );
            TermType[] interpretionResult = allExpression.interpret( context );

            if ( interpretionResult.length != 1 )
            {
                throw new InterpreterException(
                                                "ExaclyOne expression returned interpretation result unequal 1." );
            }

            result.add( interpretionResult[0] );
        }

    }

    private void interpretOneOrMoreExpression( TermCompositorType[] compositor, InterpreterContext context )
        throws InterpreterException
    {

        for ( int i = 0; i < compositor.length; i++ )
        {
            OneOrMoreExpression oneOrMoreExpression = new OneOrMoreExpression( compositor[i] );
            TermType[] interpretionResult = oneOrMoreExpression.interpret( context );

            if ( interpretionResult.length == 0 )
            {
                throw new InterpreterException( "OneOrMore expression returned no interpretation result." );
            }

            for ( int j = 0; j < interpretionResult.length; j++ )
            {
                result.add( interpretionResult[j] );
            }
        }

    }

    private void interpretTerms( TermType[] terms, InterpreterContext context ) throws InterpreterException
    {

        for ( int i = 0; i < terms.length; i++ )
        {
            TermExpression oneOrMoreExpression = new TermExpression( terms[i] );
            TermType[] interpretionResult = oneOrMoreExpression.interpret( context );

            for ( int j = 0; j < interpretionResult.length; j++ )
            {
                result.add( interpretionResult[j] );
            }
        }

    }
}
